package com.hikcreate.drv_photo_pic.impl;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.JsonObject;
import com.hikcreate.drv_photo_pic.DrvPhoto;
import com.hikcreate.service.fdfs.service.FileService;
import com.hikcreate.utils.DateUtil;
import com.hikcreate.utils.redis.RedisClientUtil;
import oracle.sql.BLOB;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.ConnectionFactory;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Table;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.sql.*;
import java.util.Date;
import java.util.zip.CRC32;


/**
 * @author lifuyi
 * @date ${Date}
 * @description
 */
@Service("drvPhotoImpl")
public class DrvPhotoImpl implements DrvPhoto {

    @Autowired
    private RedisClientUtil redisClientUtil;

    private static Logger logger = LoggerFactory.getLogger(DrvPhotoImpl.class);
    @Value("${url}")
    private  String url;
    @Value("${username}")
    private  String username;
    @Value("${password}")
    private  String password;
    @Value("${roundDay}")
    private  int roundDay;
    @Autowired
    private FileService fileService;

    private static String startTime;
    private static String endTime;
    private static String sfzmhm;
    private static BLOB zp;
    private static String gxsj;
    private static String xzqh;
    private static String xh;
    private static String flag;
    private static String rksj;
    private static String fzjg;
    private static String sfzzp;
    private static String mqzp;
    private static String bjcsbj;

    private static Configuration hbaseConf;
    private static org.apache.hadoop.hbase.client.Connection hbaseConn;
    private static TableName guizhouTablename;
    private static TableName guiyanTablename;
    private static Table guizhoutable;
    private static Table guiyantable;
    @Value("${hbase.zookeeper.property.clientPort}")
    private String hbaseAddress;
    @Value("${hbase.zookeeper.quorum}")
    private String hbaseZkQuorum;
    @Value("${hbase.master}")
    private String hbaseMaster;
    @Value("${hbase.drv_photo.table}")
    private String driverPhotoTable;
    @Value("${hbase.driverlicense.table}")
    private String driverTable;

    private static CRC32 crc = new CRC32();

    /**
     * 同步历史图片，图片同步到time时间
     *
     * @param time
     * @return
     */
    public boolean getHisDrvPhoto(String time) {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum",hbaseZkQuorum);
        hbaseConf.set("hbase.master",hbaseMaster);
        try {
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            endTime = time;
            Class.forName("oracle.jdbc.OracleDriver");
            Connection connection1 = DriverManager.getConnection(url, username, password);
            //获取最小时间
            String minTimeSql = "select  min(GXSJ) mtime from GYJG.DRV_PHOTO";
            PreparedStatement pstm1 = connection1.prepareStatement(minTimeSql);
            ResultSet resultSet1 = pstm1.executeQuery();
            while (resultSet1.next()) {
                startTime = resultSet1.getString("MTIME").substring(0, 10);
            }
            resultSet1.close();
            pstm1.close();
            connection1.close();
            //循环获取图片
            String sql;
            String lastTime;
            String rowkey;
            while (endTime.compareTo(startTime) > 0 || endTime.compareTo(startTime) == 0) {
                lastTime = DateUtil.formatDate(DateUtil.getPastDate(DateUtil.parseDate(endTime), roundDay));
                Connection connection = DriverManager.getConnection(url, username, password);
                sql = "select * from GYJG.DRV_PHOTO where to_char(GXSJ,'yyyy-mm-dd')>='" + lastTime + "' and to_char(GXSJ,'yyyy-mm-dd')<'"
                        + endTime + "' and ZP is not null and fzjg is not null and sfzmhm is not null";
                PreparedStatement pstm = connection.prepareStatement(sql);
                ResultSet resultSet = pstm.executeQuery();
                guizhouTablename=TableName.valueOf(driverPhotoTable);
                guizhoutable=hbaseConn.getTable(guizhouTablename);
                guiyanTablename=TableName.valueOf(driverTable);
                guiyantable=hbaseConn.getTable(guiyanTablename);
                while (resultSet.next()) {
                    saveToHbase(resultSet,fileService);
                }
                endTime = lastTime;
                resultSet.close();
                pstm.close();
                connection.close();
                Thread.sleep(1000);
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if(guizhoutable!=null){
                try {
                    guizhoutable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(guiyantable!=null){
                try {
                    guiyantable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(hbaseConn!=null){
                try {
                    hbaseConn.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return true;
    }

    /**
     * 每天增量同步
     */
    @Override
    public void getIncrementDrvPhoto() {
        hbaseConf = HBaseConfiguration.create();
        hbaseConf.set("hbase.zookeeper.property.clientPort", hbaseAddress);
        hbaseConf.set("hbase.zookeeper.quorum",hbaseZkQuorum);
        hbaseConf.set("hbase.master",hbaseMaster);
        try {
            Class.forName("oracle.jdbc.OracleDriver");
            Connection connection = DriverManager.getConnection(url, username, password);
            String sql="select * from GYJG.DRV_PHOTO where GXSJ between sysdate-1 and sysdate and ZP is not null and fzjg is not null and sfzmhm is not null";
            PreparedStatement pstm = connection.prepareStatement(sql);
            ResultSet resultSet = pstm.executeQuery();
            hbaseConn = ConnectionFactory.createConnection(hbaseConf);
            guizhouTablename=TableName.valueOf(driverPhotoTable);
            guizhoutable=hbaseConn.getTable(guizhouTablename);
            guiyanTablename=TableName.valueOf(driverTable);
            guiyantable=hbaseConn.getTable(guiyanTablename);
            while (resultSet.next()) {
                saveToHbase(resultSet,fileService);
            }

        }catch (Exception e) {
            e.printStackTrace();
        } finally {
            if(guizhoutable!=null){
                try {
                    guizhoutable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(guiyantable!=null){
                try {
                    guiyantable.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if(hbaseConn!=null){
                try {
                    hbaseConn.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private static void saveToHbase(ResultSet resultSet,FileService fileService) throws Exception {
        paramResultSet(resultSet);
        //存fastdfs
        String fastDfsUrl = uploadFastDfs(fileService);
        //存hbase
        //贵州省库
        //rowkey
        saveGuiZhouHbase(fastDfsUrl);
        //贵阳市库
        if("贵A".equals(fzjg)){
            String rowkey=sfzmhm;
            Put put=new Put(rowkey.getBytes());
            put.addColumn("info".getBytes(),"picUrl".getBytes(),fastDfsUrl.getBytes());
            guiyantable.put(put);
        }
    }

    public static String getPrefixString(long prefixLong) {
        if (prefixLong < 10L) {
            return "00" + prefixLong;
        } else if (prefixLong >= 10L && prefixLong < 100L) {
            return "0" + prefixLong;
        } else {
            return "" + prefixLong;
        }
    }

    private static void saveGuiZhouHbase(String fastDfsUrl) throws IOException {
        //rowkey
        String rowkey;
        crc.update(sfzmhm.getBytes());
        long prefixLong=Math.abs(crc.getValue())%300;
        String prefixString=getPrefixString(prefixLong);
        if(gxsj.length()>=19){
            rowkey=prefixString+"-"+gxsj.substring(0,19)+"-"+sfzmhm;
        }else {
            rowkey=prefixString+"-"+gxsj+"-"+sfzmhm;
        }
        Put put=new Put(rowkey.getBytes());
        put.addColumn("info".getBytes(),"idCard".getBytes(),sfzmhm.getBytes());
        put.addColumn("info".getBytes(),"picUrl".getBytes(),fastDfsUrl.getBytes());
        put.addColumn("info".getBytes(),"updateTime".getBytes(),(gxsj==null?"null":gxsj).getBytes());
        put.addColumn("info".getBytes(),"administrativeDivision".getBytes(),(xzqh==null?"null":xzqh).getBytes());
        put.addColumn("info".getBytes(),"flag".getBytes(),(flag==null?"null":flag).getBytes());
        put.addColumn("info".getBytes(),"serialNumber".getBytes(),(xh==null?"null":xh).getBytes());
        put.addColumn("info".getBytes(),"insertTime".getBytes(),(rksj==null?"null":rksj).getBytes());
        put.addColumn("info".getBytes(),"licensingAuthority".getBytes(),(fzjg==null?"null":fzjg).getBytes());
        guizhoutable.put(put);
    }

    private synchronized static String uploadFastDfs(FileService fileService) throws Exception {
        //bolb转byte
        BufferedInputStream  is = new BufferedInputStream(zp.getBinaryStream());
        byte[] bytes = new byte[(int) zp.length()];
        int len = bytes.length;
        int offset = 0;
        int read = 0;
        while (offset < len && (read = is.read(bytes, offset, len - offset)) >= 0) {
            offset += read;
        }
        //存fastdfs
        String fastDfsUrl = fileService.uploadFile(bytes, sfzmhm + gxsj + ".jpeg");
        is.close();
        return fastDfsUrl;
    }

    private synchronized static void paramResultSet(ResultSet resultSet) throws SQLException {
        sfzmhm=resultSet.getString("SFZMHM");
        zp= (BLOB) resultSet.getBlob("ZP");
        gxsj=resultSet.getString("GXSJ");
        xzqh=resultSet.getString("XZQH");
        flag=resultSet.getString("FLAG");
        xh=resultSet.getString("XH");
        rksj=resultSet.getString("RKSJ");
        fzjg=resultSet.getString("FZJG");
    }

    public static void main(String[] args) {
        String s = "{\"captureType\":\"1\",\"carColor\":\"11\",\"carLogo\":\"0\",\"carSpeed\":\"38.00\",\"conLevel\":\"?\",\"dataSource\":\"1\",\"departmentCode\":\"\",\"devCode\":\"520100000900015905\",\"directionCode\":\"4\",\"firmCode\":\"02\",\"jyw\":\"-1\",\"kafkaSource\":\"2\",\"locationDesc\":\"朝阳洞路与望城路交叉口\",\"monitorTime1\":\"2019-07-11 13:49:53.000\",\"monitorTime2\":\"?\",\"monitorTime3\":\"?\",\"monitorTime4\":\"?\",\"picFeatures\":\"\",\"picUrl1\":\"ftp://vion9:vion9@52.1.123.218:21/52.3.120.233/kk/2019-07-11/13/2019071113495366000000060.jpg\",\"picUrl2\":\"\",\"picUrl3\":\"\",\"picsNum\":\"1\",\"plateColor\":\"4\",\"plateNum\":\"无牌\",\"plateType\":\"00\",\"roadNumCode\":\"3\",\"watchTime\":\"2019-07-11 13:49:53.000\"}";
        JSONObject jsonObject = JSONObject.parseObject(new String(s));
       jsonObject.put("type", "oc");
        System.out.println(jsonObject.toJSONString());
    }

}
